@@ -67,7 +67,7 @@ static bool DoDownloadFile(CommandLine &CmdL)				/*{{{*/
 
       fileind += 2;
 
-      // An empty string counts as a hash for compatability reasons
+      // An empty string counts as a hash for compatibility reasons
       if (CmdL.FileSize() > fileind + 1 && *CmdL.FileList[fileind + 1] == '\0')
 	 fileind++;
 
@@ -133,7 +133,7 @@ static bool DoSrvLookup(CommandLine &CmdL)				/*{{{*/
    return true;
 }
 									/*}}}*/
-static const APT::Configuration::Compressor *FindCompressor(std::vector<APT::Configuration::Compressor> const & compressors, std::string name)				/*{{{*/
+static const APT::Configuration::Compressor *FindCompressor(std::vector<APT::Configuration::Compressor> const &compressors, std::string const &name) /*{{{*/
 {
    APT::Configuration::Compressor const * compressor = NULL;
    for (auto const & c : compressors)
